; Scheme 9 from Empty Space, Function Library
; By Nils M Holm, 2009,2010
; See the LICENSE file of the S9fES package for terms of use
;
; (string-find-last string1 string2)          ==>  string | #f
; (string-ci-find-last string1 string2)       ==>  string | #f
; (string-find-last-word string1 string2)     ==>  string | #f
; (string-ci-find-last-word string1 string2)  ==>  string | #f
;
; Find the last occurrence of a small string STRING1 in a large
; string STRING2. Return the last substring of STRING2 beginning
; with STRING1. When STRING2 does not contain STRING1, return #F.
; STRING-CI-FIND-LAST performs the same function, but ignores case.
;
; STRING-FIND-LAST-WORD (STRING-CI-FIND-LAST-WORD) differs from
; STRING-FIND-LAST (STRING-CI-FIND-LAST) in that is matches only
; full words, where a full word is a subsequence of characters that
; is delimited on both sides by one of the following:
;
;         - the beginning of the string;
;         - the end of the string;
;         - a non-alphabetic character.
;
; Example:   (string-find-last "a" "aaaaa")        ==>  "a"
;            (string-ci-find-last "A" "ab ac")     ==>  "ac"
;            (string-find-last "ax" "ab ac")       ==>  #f
;            (string-find-last-word "a" "ab a c")  ==>  "a c"
;            (string-find-last-word "a" "ab ac")   ==>  #f

(define (make-string-find-last p?)
  (lambda (u s)
    (let ((ks (string-length s))
          (ku (string-length u)))
      (let find ((i (- ks ku)))
        (cond ((negative? i)
                #f)
              ((p? u (substring s i (+ i ku)))
                (substring s i ks))
              (else
                (find (- i 1))))))))

(define (make-string-find-last-word p?)
  (lambda (w s)
    (let ((ks (string-length s))
          (kw (string-length w)))
      (let find ((i (- ks kw)))
        (cond ((negative? i) #f)
              ((and (p? w (substring s i (+ i kw)))
                    (or (zero? i)
                        (not (char-alphabetic?
                               (string-ref s (- i 1)))))
                    (or (= ks (+ i kw))
                        (not (char-alphabetic?
                               (string-ref s (+ i kw))))))
                (substring s i ks))
              (else
                (find (- i 1))))))))

(define string-find-last         (make-string-find-last string=?))
(define string-ci-find-last      (make-string-find-last string-ci=?))
(define string-find-last-word    (make-string-find-last-word string=?))
(define string-ci-find-last-word (make-string-find-last-word string-ci=?))
