; Scheme 9 from Empty Space, Function Library
; By Nils M Holm, 2009,2010
; See the LICENSE file of the S9fES package for terms of use
;
; (string-find string1 string2)          ==>  string | #f
; (string-ci-find string1 string2)       ==>  string | #f
; (string-find-word string1 string2)     ==>  string | #f
; (string-ci-find-word string1 string2)  ==>  string | #f
;
; Find the first occurrence of a small string STRING1 in a large
; string STRING2. Return the first substring of STRING2 beginning
; with STRING1. When STRING2 does not contain STRING1, return #F.
; STRING-CI-FIND performs the same function, but ignores case.
;
; STRING-FIND-WORD (STRING-CI-FIND-WORD) differs from STRING-FIND
; (STRING-CI-FIND) in that is matches only full words, where a full
; word is a subsequence of characters that is delimited on both
; sides by one of the following:
;
;         - the beginning of the string;
;         - the end of the string;
;         - a non-alphabetic character.
;
; Example:   (string-find "ein" "gemeinsam")     ==>  "einsam"
;            (string-find "people" "democracy")  ==>  #f
;            (string-find-word "me" "test me")   ==>  "me"
;            (string-find-word "me" "testme")    ==>  #f

(define (make-string-find p?)
  (lambda (u s)
    (let ((ks (string-length s))
          (ku (string-length u)))
      (let find ((i 0))
        (cond ((> (+ i ku) ks)
                #f)
              ((p? u (substring s i (+ i ku)))
                (substring s i ks))
              (else
                (find (+ 1 i))))))))

(define (make-string-find-word p?)
  (lambda (w s)
    (let ((ks (string-length s))
          (kw (string-length w)))
      (let find ((i 0))
        (cond ((> (+ i kw) ks) #f)
              ((and (p? w (substring s i (+ i kw)))
                    (or (zero? i)
                        (not (char-alphabetic?
                               (string-ref s (- i 1)))))
                    (or (= ks (+ i kw))
                        (not (char-alphabetic?
                               (string-ref s (+ i kw))))))
                (substring s i ks))
              (else
                (find (+ 1 i))))))))

(define string-find         (make-string-find string=?))
(define string-ci-find      (make-string-find string-ci=?))
(define string-find-word    (make-string-find-word string=?))
(define string-ci-find-word (make-string-find-word string-ci=?))
