#! /usr/local/bin/s9 -f

; s9resolve -- resolve S9fES library dependencies
; By Nils M Holm, 2010
; See the LICENSE file of the S9fES package for terms of use
;
; Usage: s9resolve [file]
;
; This program replaces LOAD-FROM-LIBRARY forms in S9fES with the
; code contained in the referenced libraries, thereby turning
; S9fES code into more portable Scheme code.
;
; Options:
;
; -s  strip initial comment blocks from included programs

(load-from-library "read-line.scm")
(load-from-library "string-split.scm")
(load-from-library "basename.scm")
(load-from-library "parse-optionsb.scm")

(define *depth* 0)

(define skip-comments (option #\s #f))
(define show-help     (option #\h #f))
(define options       `(,skip-comments
                        ,show-help))

(define (process)
  (let loop ((line (read-line)))
    (cond ((eof-object? line))
          ((let* ((tokens (string-split #\space line))
                  (first  (car tokens)))
             (and (= (string-length first) 18)
                  (string-ci=? "(load-from-library" first)))      ;) balance
            (let* ((tokens (string-split #\space line))
                   (lib    (string-split #\" (cadr tokens)))
                   (lib    (if (or (null? lib)
                                   (null? (cdr lib)))
                               (error "malformed load-from-library" line)
                               (cadr lib))))
              (cond ((or (locate-file lib)
                         (locate-file (string-append lib ".scm")))
                      => (lambda (file)
                           (newline)
                           (display "; ----- included file: ")
                           (display (basename file))
                           (display " -----")
                           (newline)
                           (newline)
                           (process-file file)
                           (display "; ----- end of file: ")
                           (display (basename file))
                           (display " -----")
                           (newline)
                           (loop (read-line))))
                    (else
                      (error "failed to resolve" lib)))))
          (else
            (display line)
            (newline)
            (loop (read-line))))))
    
(define (process-file file)
  (set! *depth* (+ 1 *depth*))
  (with-input-from-file
    file
    (lambda ()
      (if (opt-val skip-comments)
          (let loop ((line (read-line)))
            (if (and (not (eof-object? line))
                     (not (string=? "" line))
                     (char=? #\; (string-ref line 0)))
                (loop (read-line)))))
      (process)))
  (set! *depth* (- *depth* 1))
  (newline))

(define (usage)
  (display "Usage: s9scm [-s] [file ...]")
  (newline))

(let ((files (parse-options! (sys:command-line) options usage)))
  (cond ((opt-val show-help)
          (display-usage
            `(""
              ,usage
              ""
              "Resolve S9 library dependencies"
              ""
              "-s  strip initial comment blocks from included programs"
              ""))
          (sys:exit 0))
        ((null? files)
          (process))
        (else
          (map (lambda (file)
                 (newline)
                 (with-input-from-file file process))
               files))))
